import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:mask_text_input_formatter/mask_text_input_formatter.dart';
import 'Consts.dart';
import 'artemis_icons_icons.dart';

class ArtemisTextField extends StatefulWidget {
  final TextEditingController? controller;
  final TextInputType? keyboardType;
  final EdgeInsetsGeometry? padding;
  final String? label;
  final String? hint;
  final Widget? prefix;
  final Widget? suffix;
  final bool locked;
  final bool disabled;
  final bool required;
  final bool obscureText;
  final bool showConfirm;
  final bool showError;
  final int maxLines;
  final FocusNode? focusNode;
  final bool Function(String)? validator;
  final List<TextInputFormatter>? inputFormatters;
  final void Function(String value)? onSubmit;

  const ArtemisTextField(
      {Key? key,
      this.controller,
      this.showConfirm = false,
      this.onSubmit,
      this.showError = false,
      this.locked = false,
      this.disabled = false,
      this.obscureText = false,
      this.required = false,
      this.label,
      this.hint,
      this.maxLines = 1,
      this.suffix,
      this.focusNode,
      this.prefix,
      this.keyboardType,
      this.padding,
      this.inputFormatters,
      this.validator})
      : super(key: key);

  @override
  _ArtemisTextFieldState createState() => _ArtemisTextFieldState();
}

class _ArtemisTextFieldState extends State<ArtemisTextField> {
  @override
  void initState() {
    WidgetsBinding.instance.addPostFrameCallback((_) {
      if (widget.controller != null) {
        widget.controller!.addListener(() {
          if (mounted) {
            setState(() {});
          }
        });
      }
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    bool isValid = (widget.validator == null ||
        widget.validator!(
            widget.controller == null ? "" : widget.controller!.text));
    return TextField(
      enabled: !(widget.locked || widget.disabled),
      maxLines: widget.obscureText ? 1 : widget.maxLines,
      focusNode: widget.focusNode,
      obscureText: widget.obscureText,
      keyboardType: widget.keyboardType,
      inputFormatters: widget.inputFormatters,
      onSubmitted: widget.onSubmit,
      style: TextStyle(fontSize: 16),
      controller: widget.controller,
      decoration: ((isValid && widget.showConfirm)
              ? Styles.textFieldStyleConfirmed
              : (!isValid && widget.showError)
                  ? Styles.textFieldStyleInvalid
                  : widget.required
                      ? Styles.textFieldStyleRequired
                      : Styles.textFieldStyle)
          .copyWith(
              hintText: widget.hint ?? "",
              labelText: "${widget.label ?? ""}",
              labelStyle: !widget.locked
                  ? TextStyle(color: ArtemisColors.black1)
                  : TextStyle(
                      color: ArtemisColors.black1,
                      height: (widget.controller == null ||
                              widget.controller!.text.isEmpty)
                          ? 1
                          : 1),
              contentPadding: EdgeInsets.only(
                  left: widget.prefix == null ? 16 : 0, top: 16, bottom: 1),
              prefixIconConstraints: BoxConstraints(maxHeight: 50),
              disabledBorder: widget.disabled
                  ? OutlineInputBorder(
                      borderRadius: BorderRadius.circular(5),
                      borderSide: BorderSide(
                          width: 1, color: ArtemisColors.veryLightPink),
                    )
                  : null,
              prefixIcon: widget.prefix == null
                  ? null
                  : Row(
                      mainAxisSize: MainAxisSize.min,
                      children: [
                        Container(
                          padding: EdgeInsets.symmetric(horizontal: 12),
                          decoration: BoxDecoration(
                              borderRadius: BorderRadius.only(
                                topLeft: Radius.circular(5),
                                bottomLeft: Radius.circular(5),
                              ),
                              color: ArtemisColors.veryLightPink),
                          child: Center(child: widget.prefix),
                        ),
                        SizedBox(width: 16)
                      ],
                    ),
              suffixIcon: (isValid && widget.showConfirm) ||
                      (!isValid && widget.showError)
                  ? Row(
                      mainAxisAlignment: MainAxisAlignment.end,
                      mainAxisSize: MainAxisSize.min,
                      children: [
                        Padding(
                          padding: const EdgeInsets.only(right: 12),
                          child: (isValid && widget.showConfirm)
                              ? Icon(ArtemisIcons.check,
                                  size: 14, color: ArtemisColors.darkMint)
                              : (!isValid && widget.showError)
                                  ? Icon(ArtemisIcons.error,
                                      size: 14, color: ArtemisColors.red)
                                  : Container(
                                      width: 0,
                                    ),
                        ),
                        widget.locked
                            ? Padding(
                                padding: const EdgeInsets.only(right: 12),
                                child: Icon(
                                  ArtemisIcons.lock,
                                  color: ArtemisColors.black1,
                                ),
                              )
                            : Padding(
                                padding: EdgeInsets.only(
                                    right: widget.suffix == null ? 0 : 12),
                                child: widget.suffix ??
                                    Container(
                                      width: 0,
                                    ),
                              ),
                      ],
                    )
                  : widget.locked
                      ? Icon(
                          ArtemisIcons.lock,
                          color: ArtemisColors.black1,
                        )
                      : widget.suffix),
    );
  }
}

class MyTextFormatter {
  MyTextFormatter._();

  static TextInputFormatter numberFormatter = MaskTextInputFormatter(
      mask: '###,###,###,###', filter: {"#": RegExp(r'[0-9]')});
  static TextInputFormatter numberFormatter2 =
      ThousandsSeparatorInputFormatter();
}

class ThousandsSeparatorInputFormatter extends TextInputFormatter {
  static const separator = ','; // Change this to '.' for other locales

  @override
  TextEditingValue formatEditUpdate(
      TextEditingValue oldValue, TextEditingValue newValue) {
    // Short-circuit if the new value is empty
    if (newValue.text.length == 0) {
      return newValue.copyWith(text: '');
    }

    // Handle "deletion" of separator character
    String oldValueText = oldValue.text.replaceAll(separator, '');
    String newValueText = newValue.text.replaceAll(separator, '');

    if (oldValue.text.endsWith(separator) &&
        oldValue.text.length == newValue.text.length + 1) {
      newValueText = newValueText.substring(0, newValueText.length - 1);
    }

    // Only process if the old value and new value are different
    if (oldValueText != newValueText) {
      int selectionIndex =
          newValue.text.length - newValue.selection.extentOffset;
      final chars = newValueText.split('');

      String newString = '';
      for (int i = chars.length - 1; i >= 0; i--) {
        if ((chars.length - 1 - i) % 3 == 0 && i != chars.length - 1)
          newString = separator + newString;
        newString = chars[i] + newString;
      }

      return TextEditingValue(
        text: newString.toString(),
        selection: TextSelection.collapsed(
          offset: newString.length - selectionIndex,
        ),
      );
    }

    // If the new value and old value are the same, just return as-is
    return newValue;
  }
}
